/*******************************************************************************
 * Copyright (c) 2008 Mia-Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Gabriel Barbier (Mia-Software) - initial API and implementation
 *******************************************************************************/

package org.eclipse.gmt.modisco.common.editor.popup.actions;

import org.eclipse.emf.common.ui.URIEditorInput;
import org.eclipse.emf.common.util.URI;
import org.eclipse.gmt.modisco.common.editor.EcoreEditorPlugin;
import org.eclipse.gmt.modisco.common.editor.editors.EcoreEditor;
import org.eclipse.gmt.modisco.common.editor.ui.DialogUriSelection;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.ide.IDE;

/**
 * @author Gabriel Barbier
 *
 */
public class OpenModelFromEmfRegistryAction implements IObjectActionDelegate {

	private IWorkbenchPage activePage = null;

	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		this.activePage = targetPart.getSite().getPage();
	}

	public void run(IAction action) {
		
		if (this.activePage != null) {
			// initialize dialog icon and store old one
			Image oldImage = Window.getDefaultImage();
			Window.setDefaultImage(EcoreEditorPlugin.getImageDescriptor("icons/logoMoDiscoMini.jpg").createImage());
			DialogUriSelection launcher = new DialogUriSelection(new Shell());
			if (launcher.open() == Window.OK) {
				String metamodelLocation = launcher.getUriSelected();
				try {
					IEditorInput input = new URIEditorInput(URI.createURI(metamodelLocation));
					
					IDE.openEditor(this.activePage, input, EcoreEditor.EDITOR_ID, true);
				} catch (PartInitException e) {
					e.printStackTrace();
					EcoreEditorPlugin.INSTANCE.log(e);
				}
			}
			// restore default icon
			Window.setDefaultImage(oldImage);
		}
	}

	public void selectionChanged(IAction action, ISelection selection) {
		
	}

}
